@@ -3,14 +3,16 @@ import json
 import os
 
 from cirrocumulus.abstract_db import AbstractDB
-from cirrocumulus.envir import *
+from cirrocumulus.envir import CIRRO_JOB_RESULTS, SERVER_CAPABILITY_EDIT_DATASET, SERVER_CAPABILITY_ADD_DATASET, \
+    SERVER_CAPABILITY_DELETE_DATASET
 from cirrocumulus.io_util import unique_id
 from cirrocumulus.job_api import save_job_result_to_file
 from cirrocumulus.util import get_fs
 
 
 def create_dataset_meta(path):
-    result = {'id': path, 'url': path, 'name': os.path.splitext(os.path.basename(path))[0], 'description': ''}
+    result = {'id': path, 'url': path, 'name': os.path.splitext(os.path.basename(path.rstrip('/')))[0],
+              'description': ''}
     if os.path.basename(path).endswith('.json'):
         with open(path, 'rt') as f:
             result.update(json.load(f))
@@ -61,7 +63,10 @@ class LocalDbAPI(AbstractDB):
 
             if os.path.exists(json_path) and os.path.getsize(json_path) > 0:
                 with open(json_path, 'rt') as f:
-                    json_data.update(json.load(f))
+                    try:
+                        json_data.update(json.load(f))
+                    except:
+                        print('Unable to load {}'.format(json_path))
             meta = create_dataset_meta(path)
             if 'filters' not in json_data:
                 json_data['filters'] = {}
@@ -130,7 +135,9 @@ class LocalDbAPI(AbstractDB):
     def datasets(self, email):
         results = []
         for key in self.dataset_to_info:
-            results.append(self.dataset_to_info[key]['meta'])
+            m = self.dataset_to_info[key]['meta']
+            result = {'id': m['id'], 'url': m['url'], 'name': m['name'], 'description': m.get('description')}
+            results.append(result)
         return results
 
     def get_dataset(self, email, dataset_id, ensure_owner=False):
@@ -224,7 +231,7 @@ class LocalDbAPI(AbstractDB):
         return self.__get_entity(dataset_id=dataset_id, entity_id=view_id, kind='views')
 
     def upsert_dataset_view(self, email, dataset_id, view):
-        view['last_updated'] = datetime.datetime.utcnow()
+        view['last_updated'] = str(datetime.datetime.utcnow())
         if 'value' in view:
             view['value'] = json.dumps(view['value'])
         if email is not None:
@@ -236,14 +243,15 @@ class LocalDbAPI(AbstractDB):
         return dict(id=view_id, last_updated=view['last_updated'])
 
     def create_job(self, email, dataset_id, job_name, job_type, params):
-        import datetime
         job_id = unique_id()
         self.job_id_to_job[job_id] = dict(id=job_id, dataset_id=dataset_id, name=job_name, type=job_type, params=params,
-                                          status=None, submitted=datetime.datetime.utcnow())
+                                          status=None, submitted=str(datetime.datetime.utcnow()))
         return job_id
 
     def get_job(self, email, job_id, return_type):
-        job = self.job_id_to_job[job_id]
+        job = self.job_id_to_job.get(job_id)
+        if job is None:
+            return None
         if return_type == 'result':
             return job['result'] if 'result' in job else job
         elif return_type == 'status':
