@@ -38,17 +38,16 @@ export class RestDataset {
     getSelectedIdsPromise(q, cachedData) {
         q.id = this.id;
         if (this.local) {
-            if (cachedData['index'] == null) {
-                return this.getDataPromise({values: {dimensions: ['index']}}, cachedData).then(() => {
-                    const selectedIndices = Array.from(getPassingFilterIndices(cachedData, q.filter));
-                    const ids = [];
-                    const index = cachedData['index'];
-                    for (let i = 0, n = selectedIndices.length; i < n; i++) {
-                        ids.push(index[selectedIndices[i]]);
-                    }
-                    return {ids: ids};
-                });
-            }
+            const p = cachedData['index'] == null ? this.getDataPromise({values: {dimensions: ['index']}}, cachedData) : Promise.resolve();
+            return p.then(() => {
+                const selectedIndices = Array.from(getPassingFilterIndices(cachedData, q.filter));
+                const ids = [];
+                const index = cachedData['index'];
+                for (let i = 0, n = selectedIndices.length; i < n; i++) {
+                    ids.push(index[selectedIndices[i]]);
+                }
+                return {ids: ids};
+            });
         } else {
             return fetch(API + '/selected_ids',
                 {
@@ -64,13 +63,22 @@ export class RestDataset {
     }
 
     getJob(id) {
-        return fetch(API + '/job?id=' + id + '&ds_id=' + this.id, {headers: {'Authorization': 'Bearer ' + getIdToken()}}).then(response => {
+        return fetch(API + '/job?c=result&id=' + id + '&ds=' + this.id, {headers: {'Authorization': 'Bearer ' + getIdToken()}}).then(response => {
+            return response.json();
+        });
+    }
+
+    getJobParams(id) {
+        return fetch(API + '/job?c=params&id=' + id, {headers: {'Authorization': 'Bearer ' + getIdToken()}}).then(response => {
             return response.json();
         });
     }
 
     getJobStatus(id) {
-        return fetch(API + '/job_status?id=' + id + '&ds_id=' + this.id, {headers: {'Authorization': 'Bearer ' + getIdToken()}}).then(response => {
+        return fetch(API + '/job?c=status&id=' + id, {headers: {'Authorization': 'Bearer ' + getIdToken()}}).then(response => {
+            if (response.status === 404) {
+                return null;
+            }
             return response.json();
         });
     }
