@@ -1,11 +1,12 @@
-import {isPlainObject} from 'lodash';
+import {isPlainObject, isString} from 'lodash';
 import {combineReducers} from 'redux';
 import {
     ADD_DATASET,
+    ADD_TASK,
     DEFAULT_DARK_MODE,
     DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR,
     DEFAULT_DRAG_MODE,
-    DEFAULT_INTERPOLATOR,
+    DEFAULT_INTERPOLATORS,
     DEFAULT_LABEL_FONT_SIZE,
     DEFAULT_LABEL_STROKE_WIDTH,
     DEFAULT_MARKER_OPACITY,
@@ -14,6 +15,7 @@ import {
     DEFAULT_SHOW_FOG,
     DEFAULT_UNSELECTED_MARKER_OPACITY,
     DELETE_DATASET,
+    REMOVE_TASK,
     RESTORE_VIEW,
     SET_ACTIVE_FEATURE,
     SET_CATEGORICAL_NAME,
@@ -30,6 +32,8 @@ import {
     SET_DISTRIBUTION_PLOT_INTERPOLATOR,
     SET_DISTRIBUTION_PLOT_OPTIONS,
     SET_DOMAIN,
+    SET_DRAG_DIVIDER,
+    SET_DRAWER_OPEN,
     SET_EMAIL,
     SET_EMBEDDING_DATA,
     SET_EMBEDDING_LABELS,
@@ -39,13 +43,11 @@ import {
     SET_JOB_RESULT,
     SET_JOB_RESULTS,
     SET_LEGEND_SCROLL_POSITION,
-    SET_LOADING,
     SET_LOADING_APP,
     SET_MARKER_OPACITY,
     SET_MARKERS,
     SET_MESSAGE,
     SET_POINT_SIZE,
-    SET_PRIMARY_CHART_SIZE,
     SET_SAVED_DATASET_STATE,
     SET_SEARCH_TOKENS,
     SET_SELECTED_DISTRIBUTION_DATA,
@@ -56,6 +58,7 @@ import {
     SET_UNSELECTED_MARKER_OPACITY,
     SET_UNSELECTED_POINT_SIZE,
     SET_USER,
+    SET_WINDOW_SIZE,
     UPDATE_CATEGORICAL_COLOR,
     UPDATE_CATEGORICAL_NAME,
     UPDATE_DATASET
@@ -70,10 +73,9 @@ import {
     TRACE_TYPE_META_IMAGE,
     updateTraceColors
 } from '../util';
-import {updateJob} from '../DotPlotJobResultsPanel';
 
 
-const DIST_PLOT_OPTIONS = {
+export const DIST_PLOT_OPTIONS = {
     chartType: 'dotplot',
     violinScale: 'width',
     violinHeight: 100,
@@ -86,13 +88,8 @@ const DEFAULT_DIST_PLOT_OPTIONS = {
     obs: Object.assign({}, DIST_PLOT_OPTIONS, {chartType: 'violin'})
 };
 
-const DEFAULT_INTERPOLATOR_OBJ = {
-    name: DEFAULT_INTERPOLATOR,
-    value: getInterpolator(DEFAULT_INTERPOLATOR),
-    reversed: false
-};
 
-const DISTRIBUTION_PLOT_INTERPOLATOR_OBJ = {
+export const DISTRIBUTION_PLOT_INTERPOLATOR_OBJ = {
     name: DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR,
     value: getInterpolator(DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR),
     reversed: false,
@@ -100,9 +97,9 @@ const DISTRIBUTION_PLOT_INTERPOLATOR_OBJ = {
 };
 
 const DEFAULT_DISTRIBUTION_PLOT_INTERPOLATOR_OBJ = {
-    X: DISTRIBUTION_PLOT_INTERPOLATOR_OBJ,
-    modules: DISTRIBUTION_PLOT_INTERPOLATOR_OBJ,
-    obs: DISTRIBUTION_PLOT_INTERPOLATOR_OBJ
+    X: Object.assign({}, DISTRIBUTION_PLOT_INTERPOLATOR_OBJ),
+    modules: Object.assign({}, DISTRIBUTION_PLOT_INTERPOLATOR_OBJ),
+    obs: Object.assign({}, DISTRIBUTION_PLOT_INTERPOLATOR_OBJ)
 };
 const DEFAULT_PRIMARY_CHART_SIZE = {
     width: window.innerWidth - 280,
@@ -130,20 +127,35 @@ function chartSize(state = 300, action) {
 }
 
 
-function primaryChartSize(state = DEFAULT_PRIMARY_CHART_SIZE, action) {
+function panel(state = {dividerDelta: 0, drawerOpen: true, primaryChartSize: DEFAULT_PRIMARY_CHART_SIZE}, action) {
     switch (action.type) {
-        case SET_PRIMARY_CHART_SIZE:
-            return action.payload;
+        case SET_DRAG_DIVIDER:
+        case SET_DRAWER_OPEN:
+        case SET_WINDOW_SIZE:
+            let primaryChartSize = state.primaryChartSize;
+            const drawerOpen = action.type === SET_DRAWER_OPEN ? action.payload : state.drawerOpen;
+
+            if (action.type === SET_WINDOW_SIZE) {
+                const windowWidth = window.innerWidth;
+                const windowHeight = window.innerHeight;
+                const width = windowWidth - (drawerOpen ? 280 : 40);
+                const height = Math.max(300, windowHeight - 370);
+                primaryChartSize = {width: width, height: height};
+            } else if (action.type === SET_DRAG_DIVIDER) {
+                primaryChartSize = {width: primaryChartSize.width, height: action.payload};
+            }
+            return {drawerOpen: drawerOpen, primaryChartSize: primaryChartSize};
         default:
             return state;
     }
 }
 
+
 /**
  *
- * @param state Array of {value:str, type:str} where type is FEATURE_TYPE
+ * @param state Array of {id:str, type:str} where type is one of FEATURE_TYPE
  * @param action
- * @returns {*|*[]}
+ * @returns Array of search token objects
  */
 function searchTokens(state = [], action) {
     switch (action.type) {
@@ -187,12 +199,40 @@ function dataset(state = null, action) {
     switch (action.type) {
         case SET_DATASET:
             document.title = action.payload == null ? 'Cirro' : action.payload.name + ' - Cirro';
+            if (action.payload) {
+                let features = action.payload.var;
+                if (features) {
+                    if (features.length > 0 && isString(features[0])) {
+                        features = features.map(item => {
+                            return {id: item, group: ''};
+                        });
+                    }
+                    features.forEach(item => {
+                        if (item.group == null) {
+                            item.group = ''; // set default group
+                        }
+                        item.text = item.id;
+                        if (item.text.startsWith(item.group + '-')) { // hide group
+                            item.text = item.text.substring(item.group.length + 1);
+                        }
+                    });
+                    features.sort((item1, item2) => {
+                        const g = NATSORT(item1.group.toLowerCase(), item2.group.toLowerCase());
+                        if (g !== 0) {
+                            return g;
+                        }
+                        return NATSORT(item1.text.toLowerCase(), item2.text.toLowerCase());
+                    });
+                    action.payload.features = features;
+                }
+            }
             return action.payload;
         case UPDATE_DATASET:
             if (action.payload.id === state.id) { // update name, description, url
                 document.title = action.payload.name + ' - Cirro';
                 return Object.assign(state, action.payload);
             }
+            return state;
         default:
             return state;
     }
@@ -251,6 +291,7 @@ function embeddings(state = [], action) {
     }
 }
 
+
 function markers(state = [], action) {
     switch (action.type) {
         case SET_MARKERS:
@@ -364,12 +405,12 @@ function user(state = {}, action) {
 }
 
 
-function selection(state = new Set(), action) {
+function selection(state = null, action) {
     switch (action.type) {
         case SET_SELECTION:
             return action.payload;
         case SET_DATASET:
-            return new Set();
+            return null;
         default:
             return state;
     }
@@ -401,7 +442,13 @@ function featureSummary(state = {}, action) {
 function globalFeatureSummary(state = {}, action) {
     switch (action.type) {
         case SET_GLOBAL_FEATURE_SUMMARY:
-            return action.payload;
+            if (action.payload != null) {
+                for (let key in action.payload) {
+                    state[key] = action.payload[key];
+                }
+                return Object.assign({}, state);
+            }
+            return state;
         case SET_DATASET:
             return {};
         default:
@@ -416,6 +463,17 @@ function serverInfo(state = {}, action) {
                 action.payload.ontology.cellTypes.forEach(item => item.text = item.name);
                 action.payload.ontology.cellTypes.sort((item1, item2) => NATSORT(item1.text, item2.text));
             }
+            if (action.payload.datasetSelectorColumns == null) {
+                action.payload.datasetSelectorColumns = [{field: 'name', label: 'Name', visible: true}, {
+                    field: 'species',
+                    label: 'Species',
+                    visible: true
+                }, {
+                    field: 'title',
+                    label: 'Title',
+                    visible: true
+                }];
+            }
             return action.payload;
         default:
             return state;
@@ -441,13 +499,9 @@ function tab(state = 'embedding', action) {
         case SET_DATASET:
             return 'embedding';
         case SET_DISTRIBUTION_DATA:
-            if (state === 'distribution' && action.payload.length === 0) {
-                return 'embedding';
-            }
+            return (state === 'distribution' && action.payload.length === 0) ? 'embedding' : state;
         case SET_SEARCH_TOKENS:
-            if (state === 'composition' && action.payload.filter(item => item.type === FEATURE_TYPE.OBS_CAT).length < 2) {
-                return 'embedding';
-            }
+            return (state === 'composition' && action.payload.filter(item => item.type === FEATURE_TYPE.OBS_CAT).length < 2) ? 'embedding' : state;
         default:
             return state;
     }
@@ -473,6 +527,7 @@ function distributionPlotInterpolator(state = DEFAULT_DISTRIBUTION_PLOT_INTERPOL
                 interpolator.value = getInterpolator(interpolator.name);
                 return interpolator;
             }
+            return state;
         default:
             return state;
     }
@@ -496,14 +551,11 @@ function distributionPlotOptions(state = DEFAULT_DIST_PLOT_OPTIONS, action) {
 }
 
 
-function jobResult(state = null, action) {
+function jobResultId(state = null, action) {
     switch (action.type) {
         case SET_DATASET:
             return null;
         case SET_JOB_RESULT:
-            if (action.payload != null && action.payload.type == 'de') {
-                updateJob(action.payload); // initialize
-            }
             return action.payload;
         default:
             return state;
@@ -636,15 +688,16 @@ function embeddingData(state = [], action) {
         case SET_EMBEDDING_DATA :
             return action.payload;
         case SET_SELECTION:
-            state.forEach(trace => {
+            state.forEach((trace, index) => {
                 if (trace.type === TRACE_TYPE_META_IMAGE) {
                     trace.categoryToStats = action.payload.size === 0 ? trace.fullCategoryToStats : createCategoryToStats(trace, action.payload);
                     updateTraceColors(trace);
+                    state[index] = Object.assign({}, trace);
                 }
             });
             return state.slice();
         case SET_DOMAIN:
-            state.forEach((trace) => {
+            state.forEach((trace, index) => {
                 if (trace.continuous && trace.name === action.payload.name) {
                     const summary = action.payload.summary;
                     const domain = trace.type === TRACE_TYPE_META_IMAGE ? [-3, 3] : [summary.min, summary.max];
@@ -667,27 +720,29 @@ function embeddingData(state = [], action) {
 
                     trace.colorScale.domain(domain);
                     updateTraceColors(trace);
+                    state[index] = Object.assign({}, trace);
                 }
             });
             return state.slice();
         case UPDATE_CATEGORICAL_COLOR:
-            state.forEach((trace) => {
+            state.forEach((trace, index) => {
                 if (!trace.continuous && trace.name === action.payload.name) {
                     const range = trace.colorScale.range();
                     range[trace.colorScale.domain().indexOf(action.payload.originalValue)] = action.payload.color;
                     trace.colorScale.range(range);
                     updateTraceColors(trace);
+                    state[index] = Object.assign({}, trace);
                 }
             });
             return state.slice();
-
         case SET_INTERPOLATOR:
             // update colors for existing continuous traces
-            state.forEach((trace) => {
-                if (trace.continuous) {
+            state.forEach((trace, index) => {
+                if (trace.continuous && trace.featureType === action.payload.featureType) {
                     let domain = trace.colorScale.domain();
-                    trace.colorScale = createColorScale(action.payload).domain(domain);
+                    trace.colorScale = createColorScale(action.payload.value).domain(domain);
                     updateTraceColors(trace);
+                    state[index] = Object.assign({}, trace);
                 }
             });
             return state.slice();
@@ -757,22 +812,28 @@ export function legendScrollPosition(state = {}, action) {
     }
 }
 
-function loading(state = false, action) {
+// array of {label}
+function tasks(state = [], action) {
     switch (action.type) {
-        case SET_LOADING:
-            return action.payload;
+        case ADD_TASK:
+            return [...state, action.payload];
+        case REMOVE_TASK:
+            state.splice(state.indexOf(action.payload), 1);
+            return state.slice();
         default:
             return state;
     }
 }
 
 
-function interpolator(state = DEFAULT_INTERPOLATOR_OBJ, action) {
+function interpolator(state = DEFAULT_INTERPOLATORS, action) {
     switch (action.type) {
         case SET_INTERPOLATOR:
-            return action.payload;
+            const newValue = {};
+            newValue[action.payload.featureType] = action.payload.value;
+            return Object.assign({}, state, newValue);
         case RESTORE_VIEW:
-            return action.payload.colorScheme != null ? action.payload.colorScheme : state;
+            return action.payload.interpolator != null ? Object.assign({}, state, action.payload.interpolator) : state;
         default:
             return state;
     }
@@ -801,16 +862,16 @@ export default combineReducers({
     featureSummary,
     globalFeatureSummary,
     interpolator,
-    jobResult,
+    jobResultId,
     jobResults,
     legendScrollPosition,
-    loading,
+    tasks,
     loadingApp,
     markerOpacity,
     markers,
     message,
+    panel,
     pointSize,
-    primaryChartSize,
     savedDatasetState,
     searchTokens,
     selectedDistributionData,
