@@ -90,6 +90,7 @@ export const FEATURE_TYPE = {
     COUNT: 'count'
 };
 
+export const FEATURE_TYPE_MEASURES_EXCLUDE = [FEATURE_TYPE.FEATURE_SET, FEATURE_TYPE.COUNT, FEATURE_TYPE.OBS_CAT];
 export const TIES_STRATEGY_IGNORE = 0;
 export const TIES_STRATEGY_AVERAGE = 1;
 export const TIES_STRATEGY_MAXIMUM = 2;
@@ -195,7 +196,7 @@ export function summarizeDensity(values, index, selection, summarizationMethod)
     } else {
         throw new Error();
     }
-    const isSelectionEmpty = selection.size === 0;
+    const isSelectionEmpty = selection == null;
     for (let i = 0, n = index.length; i < n; i++) {
         const indices = index[i];
         const summarizedValue = summarize(isSelectionEmpty ? indices : indices.filter(item => selection.has(item)));
@@ -357,7 +358,7 @@ function getColorsRgba(trace) {
 
 
     let colorScale = trace.colorScale;
-    const npoints = trace.x.length;
+    const npoints = trace.values.length;
     const colors = new Float32Array(npoints * RGBA_NUM_ELEMENTS);
 
     function rescaleRgb(c) {
@@ -384,6 +385,7 @@ function getColorsRgba(trace) {
         const ncolors = 500;
         const colorBins = [];
         const domain = colorScale.domain();
+        const unknown = rescaleRgb(color(colorScale.unknown()));
         const binScale = scaleLinear().domain(domain).range([0, ncolors - 1]);
         for (let i = 0; i < ncolors; ++i) {
             const value = binScale.invert(i);
@@ -391,41 +393,48 @@ function getColorsRgba(trace) {
         }
         for (let i = 0, dst = 0; i < npoints; ++i) {
             const value = trace.values[i];
-            let bin = Math.ceil(binScale(value));
-            if (bin < 0) {
-                bin = 0;
-            } else if (bin >= ncolors) {
-                bin = ncolors - 1;
+            if (Number.isNaN(value)) {
+                colors[dst++] = unknown.r;
+                colors[dst++] = unknown.g;
+                colors[dst++] = unknown.b;
+                colors[dst++] = 1;
+            } else {
+                let bin = Math.ceil(binScale(value));
+                if (bin < 0) {
+                    bin = 0;
+                } else if (bin >= ncolors) {
+                    bin = ncolors - 1;
+                }
+                const c = colorBins[bin];
+                colors[dst++] = c.r;
+                colors[dst++] = c.g;
+                colors[dst++] = c.b;
+                colors[dst++] = 1;
             }
-            const c = colorBins[bin];
-            colors[dst++] = c.r;
-            colors[dst++] = c.g;
-            colors[dst++] = c.b;
-            colors[dst++] = 1;
         }
     }
 
     return colors;
 }
 
-export function updateTraceColors(traceInfo) {
-    if (traceInfo.type === TRACE_TYPE_IMAGE) {
+export function updateTraceColors(trace) {
+    if (trace.type === TRACE_TYPE_IMAGE) {
         let colors = [];
-        let colorScale = traceInfo.colorScale;
+        let colorScale = trace.colorScale;
         const colorMapper = rgb => rgb.formatHex();
-        for (let i = 0, n = traceInfo.x.length; i < n; i++) {
-            let rgb = color(colorScale(traceInfo.values[i]));
+        for (let i = 0, n = trace.values.length; i < n; i++) {
+            let rgb = color(colorScale(trace.values[i]));
             colors.push(colorMapper(rgb));
         }
-        traceInfo.colors = colors;
-    } else if (traceInfo.type === TRACE_TYPE_SCATTER) {
-        traceInfo.colors = getColorsRgba(traceInfo);
-    } else if (traceInfo.type === TRACE_TYPE_META_IMAGE) {
-        let colorScale = traceInfo.colorScale;
-        const svgNode = traceInfo.source;
-        const galleryNode = traceInfo.gallerySource;
-        const categoryToStats = traceInfo.categoryToStats;
-        if (traceInfo.name !== '__count') {
+        trace.colors = colors;
+    } else if (trace.type === TRACE_TYPE_SCATTER) {
+        trace.colors = getColorsRgba(trace);
+    } else if (trace.type === TRACE_TYPE_META_IMAGE) {
+        let colorScale = trace.colorScale;
+        const svgNode = trace.source;
+        const galleryNode = trace.gallerySource;
+        const categoryToStats = trace.categoryToStats;
+        if (trace.name !== '__count') {
             for (const category in categoryToStats) {
                 const stats = categoryToStats[category];
                 const query = category.replaceAll(' ', '_'); // FIXME
@@ -554,7 +563,7 @@ export function rankdata(values) {
 
     // Walk the sorted array, filling output array using sorted positions,
     // resolving ties as we go
-    const out = new Array(ranks.length);
+    const out = new Float32Array(ranks.length);
     let pos = 1;  // position in sorted array
     out[ranks[0].position] = pos;
     let tiesTrace = [];
@@ -697,7 +706,7 @@ export function getInterpolator(name) {
 }
 
 export function createColorScale(colorScaleDef) {
-    const scale = scaleSequential(colorScaleDef.value).clamp(true).unknown('#f0f0f0');
+    const scale = scaleSequential(colorScaleDef.value).clamp(true).unknown('#bdbdbd');
     if (colorScaleDef.reversed) {
         const interpolator = scale.interpolator();
         const mirror = t => interpolator(1 - t);
@@ -752,7 +761,7 @@ export function getFeatureSets(markers, featureSets) {
         });
 
         featureSets.forEach(featureSet => {
-            const featureSetId = isString(featureSet) ? featureSet : featureSet.value;
+            const featureSetId = isString(featureSet) ? featureSet : featureSet.id;
             const m = markerIdToMarker.get(featureSetId);
             if (m != null) {
                 filteredFeatureSets.push(m);
@@ -762,26 +771,3 @@ export function getFeatureSets(markers, featureSets) {
     return filteredFeatureSets;
 }
 
-export function splitSearchTokens(tokens) {
-    let X = [];
-    let obs = [];
-    let obsCat = [];
-    let featureSets = [];
-    let modules = [];
-    tokens.forEach(token => {
-        if (token.type === FEATURE_TYPE.X) {
-            X.push(token.value);
-        } else if (token.type === FEATURE_TYPE.OBS) {
-            obs.push(token.value);
-        } else if (token.type === FEATURE_TYPE.OBS_CAT) {
-            obsCat.push(token.value);
-        } else if (token.type === FEATURE_TYPE.FEATURE_SET) {
-            featureSets.push(token.value);
-        } else if (token.type === FEATURE_TYPE.MODULE) {
-            modules.push(token.value);
-        } else {
-            console.log('Unknown type: ' + token);
-        }
-    });
-    return {X: X, obs: obs, obsCat: obsCat, featureSets: featureSets, modules: modules, featureSetsAdd: []};
-}
